from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
import logging
import requests

from jpsecurities.site.taisyaku import Taisyaku

logger = logging.getLogger()


class KashikabuException(Exception):
    pass


class Kashikabu:
    def __init__(self, executable_path: str, chrome_options: Options,
                 implicitly_wait: int = 10, proxy=None, verify=True):
        """

        :param executable_path:
        :param chrome_options:
        :param implicitly_wait:
        :param proxy:
        :param verify:
        """
        self.executable_path = executable_path
        self.chrome_options = chrome_options
        self.implicitly_wait = implicitly_wait

        self.request = requests.Session()
        if proxy:
            self.request.proxies.update(proxy)
        self.request.verify = verify

    def get_all_list(self):
        with Taisyaku(executable_path=self.executable_path, chrome_options=self.chrome_options) as t:
            df_taisyaku = t.get_taisyaku()
        print(df_taisyaku)





import logging
from bs4 import BeautifulSoup
import requests
import pandas as pd
import sys

logger = logging.getLogger()


class KabutanException(Exception):
    pass


class Kabutan:
    base_url = "https://kabutan.jp/stock/?code="

    def __init__(self, implicitly_wait: int = 10, proxy=None, verify=True):
        """

        :param implicitly_wait:
        :param proxy:
        :param verify:
        """
        self.request = requests.Session()
        if proxy:
            self.request.proxies.update(proxy)
        self.request.verify = verify
        self.implicitly_wait = implicitly_wait

    def __enter__(self):
        """

        :return:
        """
        logger.info(f"{self.__class__.__name__} Start")
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """

        :param exc_type:
        :param exc_val:
        :param exc_tb:
        :return:
        """
        logger.info(f"{self.__class__.__name__} End")

    def get_stock_info(self, code):
        """

        :param code:
        :return:
        """
        url = f"{self.base_url}{code}"
        logging.info(f"request to {url}")
        ret = self.request.get(url, timeout=self.implicitly_wait)
        soup = BeautifulSoup(ret.content, features="html.parser")
        update_date = soup.find('div', id="kobetsu_left").findAll('time')[1].text.replace("月", "").replace("日", "")
        kobetsu_left = soup.find('div', id="kobetsu_left").findAll('tr')
        r = {"company_code": code,
             "opening_price": kobetsu_left[0].find("td").text.replace(",", ""),
             "high_price": kobetsu_left[1].find("td").text.replace(",", ""),
             "low_price": kobetsu_left[2].find("td").text.replace(",", ""),
             "closing_price": kobetsu_left[3].find("td").text.replace(",", ""),
             "trading_volume": kobetsu_left[4].find("td").text.replace("\xa0", "").replace(",", "").replace("株", ""),
             "update_date": update_date,
             "total_trading_price": kobetsu_left[5].find("td").text.replace("\xa0", ""),
             "vwap": kobetsu_left[6].find("td").text.replace("\xa0", ""),
             "execution_count": kobetsu_left[7].find("td").text.replace(" ", "").replace("回", "").replace("\xa0", ""),
             "share_unit_number": kobetsu_left[9].find("td").text.replace(",", "").replace("株", "").replace("\xa0", "")
             }
        return r



import requests
from bs4 import BeautifulSoup


class Yahoo:
    @staticmethod
    def get_stock_info(code):
        """[yahooファイナンスから株式の情報を取得]

        :param code:
        :return:
        """
        base_url = "http://stocks.finance.yahoo.co.jp/stocks/detail/"
        query = {}
        query["code"] = str(code) + ".T"
        ret = requests.get(base_url, params=query)
        soup = BeautifulSoup(ret.content)
        stocktable = soup.find('table', {'class': 'stocksTable'})
        lineFiclearfix = [list(filter(lambda x: x, s.text.split("\n"))) for s in soup.findAll('div', {'class': 'lineFi clearfix'})]
        lineFiyjMSclearfix = [list(filter(lambda x: x, s.text.split("\n"))) for s in soup.findAll('div', {'class': 'lineFi yjMS clearfix'})]
        lineFiclearfix.extend(lineFiyjMSclearfix)
        r = list(map(lambda x: {x[1]: x[0].replace("\xa0", "")}, lineFiclearfix))

        symbol = stocktable.findAll('th', {'class': 'symbol'})[0].text
        stockprice = stocktable.findAll('td', {'class': 'stoksPrice'})[1].text
        r["stock_code"] = code
        r["stock_name"] = symbol
        r["stock_price"] = stockprice
        return r


from selenium import webdriver
import logging
from jpsecurities.util.request import webdriver_to_request
from jpsecurities.util.request import download as request_download

logger = logging.getLogger()


def click_link_by_href(driver: webdriver, href_contain: str):
    """

    :param driver:
    :param href_contain:
    :return:
    """
    anchors = driver.find_elements_by_tag_name("a")
    for anchor in anchors:
        link = anchor.get_attribute("href")
        if href_contain in link:
            logger.debug(f"click_link_by_href: {link}")
            anchor.click()
            return link
    return None


def download(driver: webdriver, url: str, path: str, request_type: str = "get", post_data=None):
    """

    :param driver:
    :param url:
    :param path:
    :param request_type:
    :param post_data:
    :return:
    """
    return request_download(webdriver_to_request(driver), url, path, request_type, post_data)






import unittest
from jpsecurities.site.kabutan import Kabutan
import time
import logging
import sys
import os
from selenium.webdriver.chrome.options import Options
from logging import Formatter, handlers, StreamHandler, getLogger, DEBUG
import pandas as pd



class TestKabutan(unittest.TestCase):
    chrome_options = Options()
    chrome_options.binary_location = os.environ['CHROME_BIN_LOCATION']
    chrome_options.headless = True

    def test_get(self):
        with Kabutan() as t:
            r = t.get_stock_info("2438")
            print(r)

if __name__ == "__main__":
    unittest.main()



import unittest
from jpsecurities.site.rakuten import Rakuten
from selenium.webdriver.chrome.options import Options
import time
import logging
import sys
import os
from logging import Formatter, handlers, StreamHandler, getLogger, DEBUG

logging.getLogger().setLevel(DEBUG)
logging.getLogger().addHandler(logging.StreamHandler(sys.stdout))


class TestRakuten(unittest.TestCase):
    chrome_options = Options()
    chrome_options.binary_location = os.environ['CHROME_BIN_LOCATION']
    #chrome_options.headless = True
    executable_path = os.environ['CHROME_EXE_PATH']
    username = os.environ['RAKUTEN_USERNAME']
    pwd = os.environ['RAKUTEN_PWD']

    def test(self):
        with Rakuten(executable_path=self.executable_path, chrome_options=self.chrome_options,
                     username=self.username, pwd=self.pwd) as rakuten:
            df = rakuten.kashikabu_rate(continuous = True)
            print(df)
            df.to_csv('~/Downloads/a.csv')
            #time.sleep(10000)
            

if __name__ == "__main__":
    unittest.main()



import unittest
from jpsecurities.util.holiday import today_is_holiday, CountryType
from selenium.webdriver.chrome.options import Options
import time
import logging
import sys
import os
from logging import Formatter, handlers, StreamHandler, getLogger, DEBUG
import pandas as pd

logging.getLogger().setLevel(DEBUG)
logging.getLogger().addHandler(logging.StreamHandler(sys.stdout))


class TestKashikabu(unittest.TestCase):
    def test_is_holiday_today(self):
        print(today_is_holiday(country_type=CountryType.japan))

if __name__ == "__main__":
    unittest.main()



from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
import logging
from selenium.webdriver.support.ui import WebDriverWait
from bs4 import BeautifulSoup
from selenium.webdriver.support import expected_conditions as EC
from enum import Enum
from selenium.webdriver.common.by import By
from jpsecurities.util.selenium import click_link_by_href, download
import csv
import pandas as pd
import re
logger = logging.getLogger()
from datetime import datetime, timedelta, timezone


class GlobalMenu(Enum):
    DOMESTIC_STOCK = "国内株式"
    KASHIKABU = "貸株"
    LIST_OF_SPOT = "保有商品一覧"
    LIST_OF_MARGIN = "信用建玉一覧"


class RakutenException(Exception):
    pass


class Rakuten:
    def __init__(self, executable_path: str, chrome_options: Options, username: str, pwd: str,
                 implicitly_wait: int = 10):
        """

        :param executable_path:
        :param chrome_options:
        :return:
        """
        self.executable_path = executable_path
        self.chrome_options = chrome_options
        self.implicitly_wait = implicitly_wait
        self.username = username
        self.pwd = pwd

    def __enter__(self):
        """

        :return:
        """
        caps = DesiredCapabilities.CHROME
        caps['loggingPrefs'] = {'performance': 'INFO'}
        self.driver: webdriver = webdriver.Chrome(executable_path=self.executable_path,
                                                  chrome_options=self.chrome_options,
                                                  desired_capabilities=caps)
        self.driver.implicitly_wait(self.implicitly_wait)
        logger.info("Rakuten Start")
        self.login()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """

        :param exc_type:
        :param exc_val:
        :param exc_tb:
        :return:
        """
        self.driver.quit()
        logger.info("Rakuten End")

    def login(self):
        url = "https://www.rakuten-sec.co.jp/web/login_error.html"
        logger.info(f"Access to {url}.")
        self.driver.get(url=url)
        self.driver.find_element_by_name("loginid").send_keys(self.username)
        self.driver.find_element_by_name("passwd").send_keys(self.pwd)
        self.driver.find_element_by_name("submit%template").click()  # login

        cur_url = self.driver.current_url
        if cur_url == url:  # login failed
            logger.error(f"Rakuten login is failed.")
            raise RakutenException("Login is failed.")
        else:  # login success
            logger.info(f"Rakuten login is success.")
            return True

    def home(self, default_wait: int = 10):
        """
        ホームに移動
        :param default_wait:
        :return:
        """
        home = "pcm-g-header__inner-01"
        WebDriverWait(self.driver, default_wait).until(lambda x: x.find_element_by_class_name(home))
        self.driver.find_element_by_class_name(home).click()  # ヘッダの楽天証券ロゴをクリック
        WebDriverWait(self.driver, default_wait).until(lambda x: x.find_element_by_class_name(home))

    def get_session_id(self):
        """
        セッションIDを取得
        :return:
        """
        url = self.driver.current_url
        #qs = urllib.parse.urlparse(url).query
        #qs_d = urllib.parse.parse_qs(qs)
        logger.debug(f"get_session_id url: {url}")
        session_id = url.split("=")[1].split("?")[0]
        logger.info(f"get_session_id: {session_id}")
        return session_id

    def transition_global_menu(self, menu: GlobalMenu, default_wait: str = 10) -> BeautifulSoup:
        """
        グローバルメニューの遷移
        :param menu:
        :param default_wait:
        :return:
        """
        self.home()
        wait = WebDriverWait(self.driver, default_wait)
        result = None
        # 国内株式
        if (menu is GlobalMenu.DOMESTIC_STOCK) or (menu is GlobalMenu.KASHIKABU) or (menu is GlobalMenu.LIST_OF_SPOT) or (menu is GlobalMenu.LIST_OF_MARGIN):
            # 国内株式メニュー
            wait.until(EC.presence_of_element_located((By.LINK_TEXT, "国内株式"))).click()
            if menu is GlobalMenu.DOMESTIC_STOCK:
                result = BeautifulSoup(self.driver.page_source, "lxml")
            else:
                wait.until(EC.presence_of_element_located((By.LINK_TEXT, menu.value))).click()
                result = BeautifulSoup(self.driver.page_source, "lxml")
        return result

    def download_kashikabu_accounting_details(self, default_wait: int = 10, path: str = "/tmp"):
        """
        楽天証券の金利・配当金相当額 計上明細をダウンロード
        URL: https://member.rakuten-sec.co.jp/app/info_jp_sl_detail_list_post.do
        :param default_wait:
        :return:
        """
        wait = WebDriverWait(self.driver, default_wait)
        self.transition_global_menu(GlobalMenu.KASHIKABU, default_wait)
        wait.until(EC.presence_of_element_located((By.LINK_TEXT, "貸株"))).click()
        # 貸株詳細ページのURLを取得
        kashikabu_detail_url = click_link_by_href(self.driver, "info_jp_sl_detail_list_post")
        logging.info(f"kashikabu_detail_url: {kashikabu_detail_url}")
        # download csv
        session_id = self.get_session_id()
        download_path = f"{path}/rakuten_kashikabu_accounting_details.csv"
        kashikabu_detail_csv_rul = f"https://member.rakuten-sec.co.jp/app/info_jp_sl_detail_list_post.do;BV_SessionID={session_id}?eventType=csv"
        logging.debug(f"kashikabu_detail_csv_rul: {kashikabu_detail_csv_rul}")
        download(driver=self.driver, url=kashikabu_detail_csv_rul, path=download_path)
        return download_path

    def kashikabu_accounting_details(self, default_wait: int = 10) -> pd.DataFrame:
        """
        アカウントに紐づく楽天証券の金利・配当金相当額 計上明細を取得
        :param default_wait:
        :return:
        """
        download_path = self.download_kashikabu_accounting_details(default_wait=default_wait)
        kashikabu_accounting_details = []
        with open(download_path, 'r', encoding='shift_jis') as csvfile:
            csv_reader = csv.reader(csvfile, delimiter=',', quotechar='"')
            next(csv_reader)
            for row in csv_reader:
                kashikabu_accounting_details.append({
                    "accounting_date": row[0].replace("/", ""),
                    "jp_securities": "rakuten",
                    "service_type": row[1],
                    "stock_code": row[2],
                    "stock_name": row[3],
                    "quantity": row[4],
                    "market_price": row[5],
                    "lending_stock_interest_rate": row[6],
                    "recorded_amount": row[7]
                })
        df = pd.DataFrame(kashikabu_accounting_details)
        return df

    def download_kashikabu_rate(self, default_wait: int = 10, path: str = "/tmp"):
        """
        貸株金利一覧をダウンロード
        URL: https://member.rakuten-sec.co.jp/app/info_jp_sl_rate_search_new.do
        :param default_wait:
        :param path:
        :return:
        """
        wait = WebDriverWait(self.driver, default_wait)
        self.transition_global_menu(GlobalMenu.KASHIKABU, default_wait)
        wait.until(EC.presence_of_element_located((By.LINK_TEXT, "貸株金利"))).click()
        wait.until(EC.presence_of_element_located((By.CLASS_NAME, "tab1"))).click()
        # download csv
        session_id = self.get_session_id()
        download_path = f"{path}/rakuten_kashikabu_rate.csv"
        url = f"https://member.rakuten-sec.co.jp/app/info_jp_sl_rate_search_new.do;BV_SessionID={session_id}?eventType=csv"
        logging.debug(f"download_kashikabu_rate url : {url}")
        download(driver=self.driver, url=url, path=download_path)
        return download_path

    def kashikabu_rate(self, continuous: bool = False, default_wait: int = 10) -> pd.DataFrame:
        """

        :param continuous:
        :param default_wait:
        :return:
        """
        path = self.download_kashikabu_rate()
        df = pd.read_csv(path, encoding="SHIFT-JIS")
        df = df.rename(columns={'銘柄コード': 'コード'})
        logger.info(f"kashikabu_rate df.columns: {df.columns}")
        columns = df.columns

        # 日付のフォーマット処理
        from_date = re.findall("(?<=（).+?(?=）)", columns[2])[0]
        to_date = re.findall("(?<=（).+?(?=\-)", columns[3])[0]
        JST = timezone(timedelta(hours=+9), 'JST')
        today = datetime.now(JST).date()

        logger.info(f"rakuten kashikabu rate, from: {from_date}, to: {to_date}")
        if to_date > from_date:
            # TBD: 年跨ぎの処理
            formated_from_date = datetime.strptime(f"{today.year}/{from_date}", '%Y/%m/%d')
            formated_to_date = datetime.strptime(f"{today.year}/{to_date}", '%Y/%m/%d') - timedelta(days=1)
            df["from_date"] = formated_from_date
            df["to_date"] = formated_to_date
            df["next_from_date"] = formated_to_date + timedelta(days=1)
            df["next_to_date"] = formated_to_date + timedelta(days=7)
            df = df.rename(columns={columns[0]: 'stock_code'})
            df = df.rename(columns={columns[1]: 'stock_name'})
            df = df.rename(columns={columns[2]: 'interest_rate'})
            df = df.rename(columns={columns[3]: 'next_interest_rate'})
            df = df.rename(columns={columns[4]: 'trust_interest_rate'})
            df = df.rename(columns={columns[5]: 'next_trust_interest_rate'})
            df = df.rename(columns={columns[6]: 'non_target_period'}) # 非対象期間
            df = df.rename(columns={columns[7]: 'description'}) # 摘要
            df = df.query('interest_rate != "-"')
            df = df.query('next_interest_rate != "-"')
            df = df.query('trust_interest_rate != "-"')
            df = df.query('next_trust_interest_rate != "-"')
            df["interest_rate"] = df["interest_rate"].str.replace('%', '').astype(float) * 0.01
            df["next_interest_rate"] = df["next_interest_rate"].str.replace('%', '').astype(float) * 0.01
            df["trust_interest_rate"] = df["trust_interest_rate"].str.replace('%', '').astype(float) * 0.01
            df["next_trust_interest_rate"] = df["next_trust_interest_rate"].str.replace('%', '').astype(float) * 0.01
            logging.info(f"formated_from_date formated_to_date, {formated_from_date} {formated_to_date}")
        else:
            raise RakutenException(f"from_to date is illegal, {from_date}, {to_date}")
        # 日付データの連続化処理
        # Enhance: より効率的な値の連続化
        if continuous:
            # コード一覧を取得
            codes = df["stock_code"].unique()
            r_df = None
            count = 0
            for code in codes:
                df_code = df[df["stock_code"] == code]
                # 連続化の処理
                r = pd.date_range(start=df_code.from_date.min(), end=df_code.next_to_date.max())
                df_continuous = df_code.set_index('from_date').reindex(r).fillna(method='ffill').rename_axis('from_date').reset_index()
                # 今回、次回貸株金利を反映
                df_current = df_continuous.query('from_date <= to_date').copy()
                df_current["target_date"] = df_current["from_date"]
                df_next = df_continuous.query('from_date > to_date').copy()
                df_next["target_date"] = df_next["from_date"]
                df_next["interest_rate"] = df_next["next_interest_rate"]
                df_next["trust_interest_rate"] = df_next["next_trust_interest_rate"]
                if count > 0:
                    r_df = pd.concat([r_df, df_current, df_next])
                else:
                    r_df = pd.concat([df_current, df_next])
                count = count + 1
        else:
            r_df = df
        return r_df

    def asset_info(self, default_wait: int = 10):
        """
        楽天証券の保有資産情報を取得
        :param default_wait:
        :return:
        """
        # parse home page and get content
        self.home()
        WebDriverWait(self.driver, default_wait).until(lambda x: x.find_element_by_class_name("pcm-content"))
        soup = BeautifulSoup(self.driver.page_source, "lxml")

        # daily収益
        balance_data_actual_data = soup.find("div", {"id": "balance_data_actual_data"}).findAll("tr")
        nobr = list(map(lambda x: x.text.replace(",","").replace(" 円", ""), filter(None, map(lambda x: x.find("nobr"), balance_data_actual_data))))
        """
        国内株式
        米国株式
        中国株式
        アセアン株式
        債券
        投資信託
        外貨建MMF
        預り金
        売建予想配当金
        信用取引保証金
        国内先物OP取引証拠金
        外貨預り金合計
        信用建玉評価損益
        """
        daily_revenue = {
            "rakuten_domestic_stock": nobr[0],
            "rakuten_us_stock": nobr[1],
            "rakuten_china_stock": nobr[2],
            "rakuten_asean_stock": nobr[3],
            "rakuten_bonds": nobr[4],
            "rakuten_investment_trusts": nobr[5],
            "rakuten_foreign_currency_mmf": nobr[6],
            "rakuten_deposits": nobr[7],
            "rakuten_expected_dividends_on_sales": nobr[8],
            "rakuten_margin_transaction_deposit": nobr[9],
            "rakuten_domestic_futures_op_trading_margin": nobr[10],
            "rakuten_total_foreign_currency_deposits": nobr[11],
            "rakuten_net_credit_interest_evaluation": nobr[12]
        }
        logger.debug(daily_revenue)
        return daily_revenue

    def get_spot_transaction_info(self, default_wait: int = 10) -> pd.DataFrame:
        """

        :param default_wait:
        :return:
        """
        wait = WebDriverWait(self.driver, default_wait)
        self.transition_global_menu(GlobalMenu.LIST_OF_SPOT, default_wait)

        WebDriverWait(self.driver, default_wait).until(lambda x: x.find_element_by_id("poss-tbl-sp"))
        soup = BeautifulSoup(self.driver.page_source, "lxml")
        sp = soup.find("table", {"id": "poss-tbl-sp"}).findAll("tr", {"align": "right"})

        def __parse(x):
            stock_code = x.findAll("td", {"class": "align-L valign-M"})[0].text.replace("\n", "")
            company_name = x.findAll("td", {"class": "align-L valign-M"})[1].text.replace("\n", "")
            stock_quantity = x.find("a", {"id": "stock_detail_link"}).text.replace("\n", "").replace("\t", "")
            # 平均取得価額
            average_acquisition_price = x.findAll("td", {"class": "valign-M align-R"})[1].text.split("\n")[2]
            # 取得総額
            total_acquisition_amount = x.findAll("td", {"class": "valign-M align-R"})[1].text.split("\n")[3]
            # 現在値
            _p = list(filter(lambda y: y != "", x.find("td", {"class": "cell-05 valign-M align-R"}).text.replace("\n", "").split("\t")))
            present_value = _p[0]
            # 前日比
            day_before_ratio = _p[1]
            # 時価評価額
            _p = list(filter(lambda y: y != "",
                             x.findAll("td", {"class": "valign-M align-R"})[2].text.replace("\n", "").split("\t")))
            market_value = _p[0]
            stock_gain_loss = _p[1]
            #wait.until(EC.presence_of_element_located((By.LINK_TEXT, "詳細"))).click()
            #self.driver.back()
            return {"stock_code": stock_code, "spot_company_name": company_name, "spot_quantity": stock_quantity,
                    "spot_average_acquisition_price": average_acquisition_price,
                    "spot_total_acquisition_amount": total_acquisition_amount,
                    "spot_present_value": present_value, "spot_day_before_ratio": day_before_ratio,
                    "spot_market_value": market_value, "spot_gain_loss": stock_gain_loss}
        d = list(map(lambda x: __parse(x), sp))
        df = pd.DataFrame(d)
        df["spot_quantity"] = df["spot_quantity"].str.replace(' 株', '').str.replace(',', '').astype(float)
        df["spot_average_acquisition_price"] = df["spot_average_acquisition_price"].str.replace(' 円', '').str.replace(
            ',', '').astype(float)
        df["spot_total_acquisition_amount"] = df["spot_total_acquisition_amount"].str.replace(',', '').str.replace(' 円',
                                                                                                                   '').astype(
            float)
        df["spot_present_value"] = df["spot_present_value"].str.replace(' 円', '').str.replace(',', '').astype(float)
        df["spot_day_before_ratio"] = df["spot_day_before_ratio"].str.replace(' 円', '').str.replace(',', '').astype(
            float)
        df["spot_market_value"] = df["spot_market_value"].str.replace(' 円', '').str.replace(',', '').astype(float)
        df["spot_gain_loss"] = df["spot_gain_loss"].str.replace(' 円', '').str.replace(',', '').astype(float)
        return df

    def get_margin_transaction_info(self, default_wait: int = 10) -> pd.DataFrame:
        """

        :param default_wait:
        :return:
        """
        wait = WebDriverWait(self.driver, default_wait)
        self.transition_global_menu(GlobalMenu.LIST_OF_MARGIN, default_wait)
        wait.until(EC.presence_of_element_located((By.LINK_TEXT, "銘柄別合計"))).click()

        soup = BeautifulSoup(self.driver.page_source, "lxml")
        sp = list(map(lambda x: x.text.replace("\t","").split("\n"), soup.find("table", {"class": "ta1"}).findAll("tr")))
        sp = list(map(lambda x: list(filter(lambda y: y != "", x)), sp))

        def __perse(s):
            company_name = s[2]
            stock_code = s[3].split("\xa0")[0]
            buy_or_sell = s[4]
            margin_type = s[5]
            stock_quantity = s[7]
            stock_gain_loss = s[10]
            return {"margin_company_name": company_name, "stock_code": stock_code, "margin_buy_or_sell": buy_or_sell,
                    "margin_type": margin_type, "margin_quantity": stock_quantity, "margin_gain_loss": stock_gain_loss}
        d = list(map(lambda x: __perse(x), sp[1:]))
        df = pd.DataFrame(d)
        df["margin_quantity"] = df["margin_quantity"].str.replace(' 円', '').str.replace(',', '').astype(float)
        df["margin_gain_loss"] = df["margin_gain_loss"].str.replace(' 円', '').str.replace(',', '').astype(float)
        return df

    def get_spot_margin_transaction_info(self, default_wait: int = 10) -> pd.DataFrame:
        df_spot = self.get_spot_transaction_info(default_wait=default_wait)
        df_margin = self.get_margin_transaction_info(default_wait=default_wait)
        df = pd.merge(df_spot, df_margin, on='stock_code', how='outer')
        return df


import unittest
from jpsecurities.site.taisyaku import Taisyaku
from selenium.webdriver.chrome.options import Options
import time
import logging
import sys
import os
from logging import Formatter, handlers, StreamHandler, getLogger, DEBUG
import pandas as pd

logging.getLogger().setLevel(DEBUG)
logging.getLogger().addHandler(logging.StreamHandler(sys.stdout))


class TestTaisyaku(unittest.TestCase):
    chrome_options = Options()
    chrome_options.binary_location = os.environ['CHROME_BIN_LOCATION']
    executable_path = os.environ['CHROME_EXE_PATH']
    username = os.environ['RAKUTEN_USERNAME']
    pwd = os.environ['RAKUTEN_PWD']

    def test_download_assets(self):
        with Taisyaku(executable_path=self.executable_path, chrome_options=self.chrome_options) as t:
            #df_pcsl = t.get_pcsl()
            #df_balance = t.get_pcsl_and_balance_url()
            #df_pcsl_balance = t.get_pcsl_balance()
            #df_pcsl_balance.to_csv('/tmp/df_pcsl_balance.csv')
            df = t.get_taisyaku()
            df.to_csv('/tmp/taisyaku.csv')
            #t.get_seigenichiran()

if __name__ == "__main__":
    unittest.main()



import logging
from bs4 import BeautifulSoup
import requests
import pandas as pd
import sys
from enum import Enum

logger = logging.getLogger()

class CountryType(Enum):
    japan = "japan"


def today_is_holiday(country_type: CountryType, date: str = None):
    # http://s-proj.com/utils/checkHoliday.php?kind=h&date=20201231&opt=market
    if country_type is CountryType.japan:
        url = "http://s-proj.com/utils/checkHoliday.php?kind=h&opt=market"
        r = requests.get(url)
        if r.content == b'else':
            return False
        else:
            return True


import unittest
from jpsecurities.site.gmo import GMO
from selenium.webdriver.chrome.options import Options
import time
import logging
import sys
import os
from logging import Formatter, handlers, StreamHandler, getLogger, DEBUG

logging.getLogger().setLevel(DEBUG)
logging.getLogger().addHandler(logging.StreamHandler(sys.stdout))


class TestGMO(unittest.TestCase):
    #chrome_options = Options()
    #chrome_options.binary_location = os.environ['CHROME_BIN_LOCATION']
    # chrome_options.headless = True
    #executable_path = os.environ['CHROME_EXE_PATH']
    #username = os.environ['SBI_USERNAME']
    #pwd = os.environ['SBI_PWD']

    def test(self):
        df = GMO.kashikabu_rate(continuous=True)
        print(df)
        df.to_csv('~/Downloads/a.csv')
        # time.sleep(10000)


if __name__ == "__main__":
    unittest.main()



import requests
from selenium import webdriver
from bs4 import BeautifulSoup
from enum import Enum


class RequestType(Enum):
    post = "post"
    get = "get"


def webdriver_to_request(driver: webdriver):
    """

    :param driver:
    :return:
    """
    session = requests.session()
    for cookie in driver.get_cookies():
        session.cookies.set(cookie["name"], cookie["value"])
    return session


def download(request: requests, url: str, path: str, request_type: RequestType = RequestType.get, post_data=None):
    """

    :param request:
    :param url:
    :param path:
    :param request_type:
    :param post_data:
    :return:
    """
    if request_type is RequestType.post:
        bi = request.post(url, post_data).content
    else:
        bi = request.get(url).content
    with open(path, "wb") as f:
        f.write(bi)
    return path



import logging
from bs4 import BeautifulSoup
import requests
import pandas as pd
import sys
from datetime import datetime, timedelta, timezone, date

logger = logging.getLogger()

class GMOException(Exception):
    pass

class GMO:

    @classmethod
    def kashikabu_rate(cls, continuous: bool = False, default_wait: int = 10) -> pd.DataFrame:
        """
        """
        # request
        url = "https://www.click-sec.com/corp/guide/kabu/stock_lending/ratelist/"
        response = requests.get(url)
        soup = BeautifulSoup(response.text, 'lxml')

        # parse
        table = soup.find('table', id="search")
        lending_adapte_dates = list(map(lambda x: x.text, table.find("thead").find_all("th")[-2:]))
        logger.debug("parse finish.")

        # 期間の処理
        prev_date_from_to = lending_adapte_dates[0].split("～")
        prev_date_from = prev_date_from_to[0]
        prev_date_to = prev_date_from_to[1]
        next_date_from_to = lending_adapte_dates[1].split("～")
        next_date_from = next_date_from_to[0]
        next_date_to = next_date_from_to[1]
        companies = table.find("tbody").find_all("tr")

        results = list()
        JST = timezone(timedelta(hours=+9), 'JST')
        today = datetime.now(JST).date()

        for company in companies:
            code = company.find('div', class_="td-01-inner").text
            name = company.find('td', class_="search-col td-02").text
            prev_lending_rate = company.find(
                'td', class_="td-03").text.split("%")[0]
            next_lending_rate = company.find(
                'td', class_="td-04").text.split("%")[0]

            def __to_date(s):
                _month = s.split("/")[0]
                _date = s.split("/")[1]
                return date(today.year, int(_month), int(_date))

            r = {
                "stock_code": int(code),
                "interest_rate": float(prev_lending_rate) * 0.01,
                "from_date":  __to_date(str(prev_date_from)),
                "to_date": __to_date(str(prev_date_to)),
                "next_interest_rate": float(next_lending_rate) * 0.01,
                "next_from_date": __to_date(str(next_date_from)),
                "next_to_date": __to_date(str(next_date_to))
            }
            if r["to_date"] < r["from_date"]:
                # TBD: 年跨ぎの処理
                raise GMOException(f'from to date is illegal,{r["to_date"]}, {r["from_date"]}')
            results.append(r)
            df = pd.DataFrame(results)
        if continuous:
            # コード一覧を取得
            codes = df["stock_code"].unique()
            r_df = None
            count = 0
            for code in codes:
                df_code = df[df["stock_code"] == code]
                # 連続化の処理
                r = pd.date_range(start=df_code.from_date.min(), end=df_code.next_to_date.max())
                df_continuous = df_code.set_index('from_date').reindex(r).fillna(method='ffill').rename_axis('from_date').reset_index()
                # format date => fillを行うとdatetimeがobject型に変換される
                #print(df_continuous.dtypes)
                _date_format = '%Y%m%d'
                df_continuous["from_date"] = df_continuous["from_date"].dt.strftime(_date_format)
                df_continuous["to_date"] = df_continuous["to_date"].astype(str).str.replace("-", "")
                df_continuous["next_from_date"] = df_continuous["next_from_date"].astype(str).str.replace("-", "")
                df_continuous["next_to_date"] = df_continuous["next_to_date"].astype(str).str.replace("-", "")
                # 今回、次回貸株金利を反映
                df_current = df_continuous.query('from_date <= to_date').copy()
                df_current["target_date"] = df_current["from_date"]
                df_next = df_continuous.query('from_date > to_date').copy()
                df_next["target_date"] = df_next["from_date"]
                df_next["interest_rate"] = df_next["next_interest_rate"]
                _r_df = pd.concat([df_current, df_next])
                if count > 0:
                    r_df = pd.concat([r_df, _r_df])
                else:
                    r_df = _r_df
                count = count + 1
        else:
            r_df = df
        return r_df



import unittest
from jpsecurities.site.yahoo import Yahoo
from selenium.webdriver.chrome.options import Options
import time
import logging
import sys
import os
from logging import Formatter, handlers, StreamHandler, getLogger, DEBUG
import pandas as pd

logging.getLogger().setLevel(DEBUG)
logging.getLogger().addHandler(logging.StreamHandler(sys.stdout))


class TestYahoo(unittest.TestCase):
    def test(self):
        print(Yahoo.get_stock_info("2438"))


if __name__ == "__main__":
    unittest.main()


import unittest
from jpsecurities.util.kasikabu import Kashikabu
from selenium.webdriver.chrome.options import Options
import time
import logging
import sys
import os
from logging import Formatter, handlers, StreamHandler, getLogger, DEBUG
import pandas as pd

logging.getLogger().setLevel(DEBUG)
logging.getLogger().addHandler(logging.StreamHandler(sys.stdout))


class TestKashikabu(unittest.TestCase):
    chrome_options = Options()
    chrome_options.binary_location = os.environ['CHROME_BIN_LOCATION']
    chrome_options.headless = True
    executable_path = os.environ['CHROME_EXE_PATH']
    username = os.environ['RAKUTEN_USERNAME']
    pwd = os.environ['RAKUTEN_PWD']

    def test_all_list(self):
        kashikabu = Kashikabu(executable_path=self.executable_path, chrome_options=self.chrome_options)
        df = kashikabu.get_all_list()
        print(df)

if __name__ == "__main__":
    unittest.main()



from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
import logging
from bs4 import BeautifulSoup
from jpsecurities.util.selenium import download
from jpsecurities.util.request import download
import requests
import pandas as pd


logger = logging.getLogger()


class TaisyakuException(Exception):
    pass


class Taisyaku:
    def __init__(self, executable_path: str, chrome_options: Options,
                 implicitly_wait: int = 10, proxy=None, verify=True):
        """

        :param executable_path:
        :param chrome_options:
        :param implicitly_wait:
        :param proxy:
        :param verify:
        """
        self.executable_path = executable_path
        self.chrome_options = chrome_options
        self.implicitly_wait = implicitly_wait

        self.request = requests.Session()
        if proxy:
            self.request.proxies.update(proxy)
        self.request.verify = verify

    def __enter__(self):
        caps = DesiredCapabilities.CHROME
        caps['loggingPrefs'] = {'performance': 'INFO'}
        self.driver: webdriver = webdriver.Chrome(executable_path=self.executable_path,
                                                  chrome_options=self.chrome_options,
                                                  desired_capabilities=caps)
        self.driver.implicitly_wait(self.implicitly_wait)
        logger.info(f"{self.__class__.__name__} Start")
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.driver.quit()
        logger.info(f"{self.__class__.__name__} End")

    def get_pcsl_and_balance_url(self):
        """
        品貸料率一覧表,銘柄別残高一覧表のダウンロードURLを取得
        :return:
        """
        url = "https://www.taisyaku.jp/"
        self.driver.get(url)
        html = self.driver.page_source.encode('utf-8')
        soup = BeautifulSoup(html, "lxml")
        raw_url = soup.find("div", {"class": "download-inner"}).findAll("a")
        urls = {
            "pcsl": raw_url[0]['href'],
            "balance": raw_url[1]['href']
        }
        return urls

    def get_balance(self, only_tosho: bool = True):
        """
        銘柄別残高一覧を取得
        :return:
        """
        name = "balance"
        url = self.get_pcsl_and_balance_url()[name]
        path = download(request=self.request, url=url, path=f"/tmp/{name}.csv")
        df = pd.read_csv(path, skiprows=4, encoding="SHIFT-JIS")
        if only_tosho:
            df = df.query('市場区分 == "東証およびＰＴＳ"')
        return df

    def get_pcsl(self, only_tosho: bool = True):
        """
        品貸料率一覧を取得
        :return:
        """
        name = "pcsl"
        url = self.get_pcsl_and_balance_url()[name]
        path = download(request=self.request, url=url, path=f"/tmp/{name}.csv")
        df = pd.read_csv(path, skiprows=4, encoding="SHIFT-JIS")
        if only_tosho:
            df = df.query('市場区分 == "東証"')
        return df

    def get_pcsl_balance(self):
        """

        :return:
        """
        df_pcsl = self.get_pcsl()
        df_pcsl["貸借申込日"] = df_pcsl["貸借申込日"].astype(str)
        df_pcsl["決済日"] = df_pcsl["決済日"].astype(str)
        df_pcsl = df_pcsl.drop("市場区分", axis=1)
        df_pcsl = df_pcsl.drop("銘柄名", axis=1)

        df_balance = self.get_balance()
        df_balance["申込日"] = df_balance["申込日"].str.replace('/','')

        df = pd.merge(df_pcsl, df_balance, on='コード', how='outer')
        return df

    def get_other(self):
        """
        貸借銘柄等一覧
        :return:
        """
        url = "https://www.taisyaku.jp/sys-list/data/other.xlsx"
        path = download(request=self.request, url=url, path=f"/tmp/other.xlsx")
        df = pd.read_excel(path, skiprows=5, engine='openpyxl')
        df = df.add_prefix("貸借取引対象銘_")
        df = df.rename(columns={'貸借取引対象銘_コード': 'コード'})
        df = df.rename(columns={'貸借取引対象銘_銘柄名': '銘柄名'})
        return df

    def get_seigenichiran(self):
        """
        注意喚起および申込停止措置等一覧表
        :return:
        """
        url = "https://www.taisyaku.jp/sys-list/data/seigenichiran.xlsx"
        path = download(request=self.request, url=url, path=f"/tmp/seigenichiran.xlsx")
        df = pd.read_excel(path, skiprows=9, engine='openpyxl')
        df = df.rename(columns={'Unnamed: 0': '直近発表'})
        df = df.rename(columns={'Unnamed: 1': 'コード'})
        df = df.rename(columns={'Unnamed: 2': '銘柄名'})
        df = df.rename(columns={'Unnamed: 3': '実施措置'})
        df = df.rename(columns={'Unnamed: 4': '実施内容'})
        df = df.rename(columns={'Unnamed: 5': '備考'})
        df = df.rename(columns={'Unnamed: 6': '通知日'})
        df = df.rename(columns={'Unnamed: 7': '実施'})
        df["通知日"] = df["通知日"].str.replace('月|日|年', '')
        df = df.add_prefix("注意喚起および申込停止措置_")
        df = df.rename(columns={'注意喚起および申込停止措置_コード': 'コード'})
        df = df.rename(columns={'注意喚起および申込停止措置_銘柄名': '銘柄名'})
        return df

    def get_other_seigenichiran(self):
        df_other = self.get_other()
        df_seigenichiran = self.get_seigenichiran()
        df_seigenichiran = df_seigenichiran.drop("銘柄名", axis=1)
        df = pd.merge(df_other, df_seigenichiran, on='コード', how='outer')
        return df

    def get_taisyaku(self, only_taisyaku_enable: bool = True):
        df_pcsl_balance = self.get_pcsl_balance()
        df_other_seigenichiran =self.get_other_seigenichiran()
        df_other_seigenichiran = df_other_seigenichiran.drop("銘柄名", axis=1)
        df = pd.merge(df_pcsl_balance, df_other_seigenichiran, on='コード', how='outer')
        return df



from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
import logging
from selenium.webdriver.support.ui import WebDriverWait
from bs4 import BeautifulSoup
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By
from jpsecurities.util.request import *
import pandas as pd

logger = logging.getLogger()
from selenium.webdriver.support import expected_conditions

class SBIException(Exception):
    pass


class SBI:
    def __init__(self, executable_path: str, chrome_options: Options, username: str, pwd: str,
                 implicitly_wait: int = 10):
        """

        :param executable_path:
        :param chrome_options:
        :return:
        """
        self.executable_path = executable_path
        self.chrome_options = chrome_options
        self.implicitly_wait = implicitly_wait
        self.username = username
        self.pwd = pwd

    def __enter__(self):
        """

        :return:
        """
        caps = DesiredCapabilities.CHROME
        caps['loggingPrefs'] = {'performance': 'INFO'}
        self.driver: webdriver = webdriver.Chrome(executable_path=self.executable_path,
                                                  chrome_options=self.chrome_options,
                                                  desired_capabilities=caps)
        self.driver.implicitly_wait(self.implicitly_wait)
        logger.info("Rakuten Start")
        self.login()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """

        :param exc_type:
        :param exc_val:
        :param exc_tb:
        :return:
        """
        self.driver.quit()
        logger.info("Rakuten End")

    def login(self):
        url = "https://site3.sbisec.co.jp/ETGate/"
        self.driver.get(url)
        wait = WebDriverWait(self.driver, self.implicitly_wait)
        wait.until(expected_conditions.element_to_be_clickable((By.ID, "user_input")))
        logger.info("SBIWebDriver.login: 1st wait_rendering finish")
        self.driver.find_element_by_name("user_id").send_keys(self.username)
        self.driver.find_element_by_name("user_password").send_keys(self.pwd)
        self.driver.find_element_by_name("ACT_login").click()  # ACT_login click
        wait.until(EC.presence_of_element_located((By.ID, "scsbaccnt-cfd_unopen")))
        logger.info("SBIWebDriver.login: login finish")
        return True

    def kashikabu_rate(self, default_wait: int = 10, tmp_path: str = "/tmp") -> pd.DataFrame:
        """

        :param default_wait:
        :param tmp_path:
        :return:
        """
        # 貸株遷移用ページ
        wait = WebDriverWait(self.driver, default_wait)
        kashikabu_init_page_url = "https://site3.sbisec.co.jp/ETGate/?OutSide=on&_ControlID=WPLETmgR001Control&_PageID=WPLETmgR001Mdtl20&_DataStoreID=DSWPLETmgR001Control&_ActionID=DefaultAID&getFlg=on&burl=search_domestic&cat1=domestic&cat2=stocklending&dir=stocklending&file=domestic_stocklending.html"
        self.driver.get(kashikabu_init_page_url)
        wait.until(EC.presence_of_element_located((By.ID, "scsbaccnt-cfd_unopen")))
        # 貸株ページ
        kashikabu_page_url = "https://site3.sbisec.co.jp/ETGate/?OutSide=on&_ControlID=WPLETsmR001Control&_DataStoreID=DSWPLETsmR001Control&_PageID=WPLETsmR001Sdtl12&_ActionID=NoActionID&sw_page=LndStk&cat1=home&cat2=none&sw_param1=&sw_param2=lndstk_ratelist&getFlg=on"
        self.driver.get(kashikabu_page_url)
        wait.until(EC.presence_of_element_located((By.ID, "footer01P")))
        # 貸株CSVページからcsvをダウンロード
        kashikabu_csv_url = "https://site0.sbisec.co.jp/marble/account/japan/ratelist/csv.do"
        request_session = webdriver_to_request(self.driver)
        post_data = {
            'init': 'false',
            'inqType': '2',
            'searchStrings': '',
            'interestSearchType': '0',
            'productCodeSearchType': '',
            'attribute': 'lriCouponRate',
            'direction': '2',
            'page': '0',
            'dispRow': '50'
        }
        download_path = f"{tmp_path}/sbi_kashikabu_rate.csv"
        download(request=request_session, url=kashikabu_csv_url, path=download_path, request_type=RequestType.post, post_data=post_data)
        df = pd.read_csv(download_path, encoding="SHIFT-JIS", skiprows=3)
        # 適用日	銘柄コード	銘柄名	適用金利（年）	非対象期間	事由
        columns = df.columns
        df = df.rename(columns={columns[0]: 'target_date'})
        df = df.rename(columns={columns[1]: 'stock_code'})
        df = df.rename(columns={columns[2]: 'stock_name'})
        df = df.rename(columns={columns[3]: 'interest_rate'})
        df = df.rename(columns={columns[4]: 'non_target_duration'})
        df = df.rename(columns={columns[5]: 'description'})
        df.target_date = df.target_date.str.replace("/", "")
        # 13.25%
        #df.interest_rate = df.query('interest_rate != ""')
        df.interest_rate = df.interest_rate.str.replace(" ", "").str.replace("-", "").str.replace("%", "") #astype(float) * 0.01
        df = df.query('interest_rate != ""')
        df.interest_rate = df.interest_rate.astype(float) * 0.01
        return df